@@ -1,7 +1,7 @@
 /* mpfr_dump -- Dump a float to stdout.
 
-Copyright 1999, 2001, 2004 Free Software Foundation, Inc.
-(Copied from the GNU MP Library.)
+Copyright 1999, 2001, 2004, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -17,10 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
-#include <stdio.h>
 #include <string.h>
 
 #include "mpfr-impl.h"
@@ -28,6 +27,6 @@ MA 02111-1307, USA. */
 void
 mpfr_dump (mpfr_srcptr u)
 {
-  mpfr_print_binary(u); 
+  mpfr_print_binary(u);
   putchar('\n');
 }
