@@ -1,6 +1,7 @@
 /* mpfr_lngamma -- lngamma function
 
-Copyright 2005 Free Software Foundation.
+Copyright 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,8 +17,8 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #define MPFR_NEED_LONGLONG_H
 #include "mpfr-impl.h"
@@ -107,6 +108,34 @@ mpfr_gamma_alpha (mp_prec_t p)
     return 0.26 * (double) MPFR_INT_CEIL_LOG2 ((unsigned long) p);
 }
 
+#ifndef IS_GAMMA
+static int
+unit_bit (mpfr_srcptr (x))
+{
+  mp_exp_t expo;
+  mp_prec_t prec;
+  mp_limb_t x0;
+
+  expo = MPFR_GET_EXP (x);
+  if (expo <= 0)
+    return 0;  /* |x| < 1 */
+
+  prec = MPFR_PREC (x);
+  if (expo > prec)
+    return 0;  /* y is a multiple of 2^(expo-prec), thus an even integer */
+
+  /* Now, the unit bit is represented. */
+
+  prec = ((prec - 1) / BITS_PER_MP_LIMB + 1) * BITS_PER_MP_LIMB - expo;
+  /* number of represented fractional bits (including the trailing 0's) */
+
+  x0 = *(MPFR_MANT (x) + prec / BITS_PER_MP_LIMB);
+  /* limb containing the unit bit */
+
+  return (x0 >> (prec % BITS_PER_MP_LIMB)) & 1;
+}
+#endif
+
 /* lngamma(x) = log(gamma(x)).
    We use formula [6.1.40] from Abramowitz&Stegun:
    lngamma(z) = (z-1/2)*log(z) - z + 1/2*log(2*Pi)
@@ -117,12 +146,12 @@ mpfr_gamma_alpha (mp_prec_t p)
    For z real, |K(z)| <= 1 thus R_n(z) is bounded by the first neglected term.
  */
 #ifdef IS_GAMMA
-static int
 #define GAMMA_FUNC mpfr_gamma_aux
 #else
-int
-#define GAMMA_FUNC mpfr_lngamma
+#define GAMMA_FUNC mpfr_lngamma_aux
 #endif
+
+static int
 GAMMA_FUNC (mpfr_ptr y, mpfr_srcptr z0, mp_rnd_t rnd)
 {
   mp_prec_t precy, w; /* working precision */
@@ -136,49 +165,77 @@ GAMMA_FUNC (mpfr_ptr y, mpfr_srcptr z0, mp_rnd_t rnd)
   double d;
   MPFR_SAVE_EXPO_DECL (expo);
 
-#ifndef IS_GAMMA
-  /* special cases */
-  if (MPFR_UNLIKELY (MPFR_IS_SINGULAR (z0)))
-    {
-      if (MPFR_IS_NAN (z0) || MPFR_IS_NEG (z0))
-        {
-          MPFR_SET_NAN (y);
-          MPFR_RET_NAN;
-        }
-      else /* lngamma(+Inf) = lngamma(+0) = +Inf */
-        {
-          MPFR_SET_INF (y);
-          MPFR_SET_POS (y);
-          MPFR_RET (0);  /* exact */
-        }
-    }
+  compared = mpfr_cmp_ui (z0, 1);
 
-  /* if x < 0 and -2k-1 <= x <= -2k, then lngamma(x) = NaN */
-  if (MPFR_IS_NEG(z0) && ((mpfr_get_si (z0, GMP_RNDZ) % 2) == 0
-                          || mpfr_integer_p (z0)))
+  MPFR_SAVE_EXPO_MARK (expo);
+
+#ifndef IS_GAMMA /* lngamma or lgamma */
+  if (compared == 0 || (compared > 0 && mpfr_cmp_ui (z0, 2) == 0))
+    return mpfr_set_ui (y, 0, GMP_RNDN);  /* lngamma(1 or 2) = +0 */
+
+  /* Deal here with tiny inputs. We have for -0.3 <= x <= 0.3:
+     - log|x| - gamma*x <= log|gamma(x)| <= - log|x| - gamma*x + x^2 */
+  if (MPFR_EXP(z0) <= - (mp_exp_t) MPFR_PREC(y))
     {
-      MPFR_SET_NAN (y);
-      MPFR_RET_NAN;
+      mpfr_t l, h, g;
+      int ok, inex2;
+
+      mpfr_init2 (l, MPFR_PREC(y) + 14);
+      if (MPFR_IS_POS(z0))
+        {
+          mpfr_log (l, z0, GMP_RNDU); /* upper bound for log(z0) */
+          mpfr_init2 (h, MPFR_PREC(l));
+        }
+      else
+        {
+          mpfr_init2 (h, MPFR_PREC(z0));
+          mpfr_neg (h, z0, GMP_RNDN); /* exact */
+          mpfr_log (l, h, GMP_RNDU); /* upper bound for log(-z0) */
+          mpfr_set_prec (h, MPFR_PREC(l));
+        }
+      mpfr_neg (l, l, GMP_RNDD); /* lower bound for -log(|z0|) */
+      mpfr_set (h, l, GMP_RNDD); /* exact */
+      mpfr_nextabove (h); /* upper bound for -log(|z0|), avoids two calls to
+                             mpfr_log */
+      mpfr_init2 (g, MPFR_PREC(l));
+      /* if z0>0, we need an upper approximation of Euler's constant
+         for the left bound */
+      mpfr_const_euler (g, MPFR_IS_POS(z0) ? GMP_RNDU : GMP_RNDD);
+      mpfr_mul (g, g, z0, GMP_RNDD);
+      mpfr_sub (l, l, g, GMP_RNDD);
+      mpfr_const_euler (g, MPFR_IS_POS(z0) ? GMP_RNDD : GMP_RNDU); /* cached */
+      mpfr_mul (g, g, z0, GMP_RNDU);
+      mpfr_sub (h, h, g, GMP_RNDD);
+      mpfr_mul (g, z0, z0, GMP_RNDU);
+      mpfr_add (h, h, g, GMP_RNDU);
+      inexact = mpfr_prec_round (l, MPFR_PREC(y), rnd);
+      inex2 = mpfr_prec_round (h, MPFR_PREC(y), rnd);
+      /* Caution: we not only need l = h, but both inexact flags should agree.
+         Indeed, one of the inexact flags might be zero. In that case if we
+         assume lngamma(z0) cannot be exact, the other flag should be correct.
+         We are conservative here and request that both inexact flags agree. */
+      ok = SAME_SIGN (inexact, inex2) && mpfr_cmp (l, h) == 0;
+      if (ok)
+        mpfr_set (y, h, rnd); /* exact */
+      mpfr_clear (l);
+      mpfr_clear (h);
+      mpfr_clear (g);
+      if (ok)
+        {
+          MPFR_SAVE_EXPO_FREE (expo);
+          return mpfr_check_range (y, inexact, rnd);
+        }
     }
 #endif
 
   precy = MPFR_PREC(y);
 
-  compared = mpfr_cmp_ui (z0, 1);
-
-#ifndef IS_GAMMA
-  if (compared == 0) /* lngamma(1) = +0 */
-    return mpfr_set_ui (y, 0, GMP_RNDN);
-#endif
-
   mpfr_init2 (s, MPFR_PREC_MIN);
   mpfr_init2 (t, MPFR_PREC_MIN);
   mpfr_init2 (u, MPFR_PREC_MIN);
   mpfr_init2 (v, MPFR_PREC_MIN);
   mpfr_init2 (z, MPFR_PREC_MIN);
 
-  MPFR_SAVE_EXPO_MARK (expo);
-
   if (compared < 0)
     {
       mp_exp_t err_u;
@@ -190,42 +247,62 @@ GAMMA_FUNC (mpfr_ptr y, mpfr_srcptr z0, mp_rnd_t rnd)
       w = precy + MPFR_INT_CEIL_LOG2 (precy);
       while (1)
         {
-          w += MPFR_INT_CEIL_LOG2 (w) + 13;
+          w += MPFR_INT_CEIL_LOG2 (w) + 14;
           MPFR_ASSERTD(w >= 3);
           mpfr_set_prec (s, w);
           mpfr_set_prec (t, w);
           mpfr_set_prec (u, w);
           mpfr_set_prec (v, w);
-          mpfr_ui_sub (s, 2, z0, GMP_RNDD); /* s = (2-z0) * (1+2u) */
-          mpfr_const_pi (t, GMP_RNDN);      /* t = Pi * (1+u) */
+          /* In the following, we write r for a real of absolute value
+             at most 2^(-w). Different instances of r may represent different
+             values. */
+          mpfr_ui_sub (s, 2, z0, GMP_RNDD); /* s = (2-z0) * (1+2r) >= 1 */
+          mpfr_const_pi (t, GMP_RNDN);      /* t = Pi * (1+r) */
           mpfr_lngamma (u, s, GMP_RNDN); /* lngamma(2-x) */
-          /* Let s = (2-z0) + h. By construction, -(2-z0)*(2u) <= h <= 0.
+          /* Let s = (2-z0) + h. By construction, -(2-z0)*2^(1-w) <= h <= 0.
              We have lngamma(s) = lngamma(2-z0) + h*Psi(z), z in [2-z0+h,2-z0].
-             Since 2-z0+h >= 1 and |Psi(x)| <= max(1,log(x)) for x >= 1,
+             Since 2-z0+h = s >= 1 and |Psi(x)| <= max(1,log(x)) for x >= 1,
              the error on u is bounded by
-             ulp(u)/2 + (2-z0)*max(1,log(2-z0))*2^(1-w). */
+             ulp(u)/2 + (2-z0)*max(1,log(2-z0))*2^(1-w)
+             = (1/2 + (2-z0)*max(1,log(2-z0))*2^(1-E(u))) ulp(u) */
           d = (double) MPFR_GET_EXP(s) * 0.694; /* upper bound for log(2-z0) */
           err_u = MPFR_GET_EXP(s) + __gmpfr_ceil_log2 (d) + 1 - MPFR_GET_EXP(u);
           err_u = (err_u >= 0) ? err_u + 1 : 0;
           /* now the error on u is bounded by 2^err_u ulps */
 
-          mpfr_mul (s, s, t, GMP_RNDN); /* Pi*(2-x), (1+u)^4 */
+          mpfr_mul (s, s, t, GMP_RNDN); /* Pi*(2-x) * (1+r)^4 */
           err_s = MPFR_GET_EXP(s); /* 2-x <= 2^err_s */
           mpfr_sin (s, s, GMP_RNDN); /* sin(Pi*(2-x)) */
-          /* the error on s is bounded by 1/2*ulp(s) + [(1+u)^4-1]*(2-x)
-             <= 1/2*ulp(s) + 5*2^(-w)*(2-x) for w >= 3 */
+          /* the error on s is bounded by 1/2*ulp(s) + [(1+2^(-w))^4-1]*(2-x)
+             <= 1/2*ulp(s) + 5*2^(-w)*(2-x) for w >= 3
+             <= (1/2 + 5 * 2^(-E(s)) * (2-x)) ulp(s) */
           err_s += 3 - MPFR_GET_EXP(s);
           err_s = (err_s >= 0) ? err_s + 1 : 0;
-          /* the error on s is bounded by 2^err_s ulps, thus the relative
-             error is bounded by 2^(err_s+1) */
-          err_s ++; /* relative error */
+          /* the error on s is bounded by 2^err_s ulp(s), thus by
+             2^(err_s+1)*2^(-w)*|s| since ulp(s) <= 2^(1-w)*|s|.
+             Now n*2^(-w) can always be written |(1+r)^n-1| for some
+             |r|<=2^(-w), thus taking n=2^(err_s+1) we see that
+             |S - s| <= |(1+r)^(2^(err_s+1))-1| * |s|, where S is the
+             true value.
+             In fact if ulp(s) <= ulp(S) the same inequality holds for
+             |S| instead of |s| in the right hand side, i.e., we can
+             write s = (1+r)^(2^(err_s+1)) * S.
+             But if ulp(S) < ulp(s), we need to add one ``bit'' to the error,
+             to get s = (1+r)^(2^(err_s+2)) * S. This is true since with
+             E = n*2^(-w) we have |s - S| <= E * |s|, thus
+             |s - S| <= E/(1-E) * |S|.
+             Now E/(1-E) is bounded by 2E as long as E<=1/2,
+             and 2E can be written (1+r)^(2n)-1 as above.
+          */
+          err_s += 2; /* exponent of relative error */
 
-          mpfr_sub_ui (v, z0, 1, GMP_RNDN); /* v = (x-1) * (1+u) */
-          mpfr_mul (v, v, t, GMP_RNDN); /* v = Pi*(x-1) * (1+u)^3 */
+          mpfr_sub_ui (v, z0, 1, GMP_RNDN); /* v = (x-1) * (1+r) */
+          mpfr_mul (v, v, t, GMP_RNDN); /* v = Pi*(x-1) * (1+r)^3 */
           mpfr_div (v, v, s, GMP_RNDN); /* Pi*(x-1)/sin(Pi*(2-x)) */
-          /* (1+u)^(4+2^err_s+1) */
-          err_s = (err_s <= 2) ? 3 + (err_s / 2) : err_s + 1;
-          MPFR_ASSERTD(MPFR_IS_POS(v));
+          mpfr_abs (v, v, GMP_RNDN);
+          /* (1+r)^(3+2^err_s+1) */
+          err_s = (err_s <= 1) ? 3 : err_s + 1;
+          /* now (1+r)^M with M <= 2^err_s */
           mpfr_log (v, v, GMP_RNDN);
           /* log(v*(1+e)) = log(v)+log(1+e) where |e| <= 2^(err_s-w).
              Since |log(1+e)| <= 2*e for |e| <= 1/4, the error on v is
@@ -270,6 +347,7 @@ GAMMA_FUNC (mpfr_ptr y, mpfr_srcptr z0, mp_rnd_t rnd)
       mpfr_set_prec (s, 53);
       /* we need z >= w*log(2)/(2*Pi) to get an absolute error less than 2^(-w)
          but the optimal value is about 0.155665*w */
+      /* FIXME: replace double by mpfr_t types. */
       mpfr_set_d (s, mpfr_gamma_alpha (precy) * (double) w, GMP_RNDU);
       if (mpfr_cmp (z0, s) < 0)
         {
@@ -299,12 +377,12 @@ GAMMA_FUNC (mpfr_ptr y, mpfr_srcptr z0, mp_rnd_t rnd)
          Since w >= 2 and z0+k >= 4, we can write log((z0+k)*(1+t1))
          = log(z0+k) * (1+t3) with |t3| <= 2^(-w), thus we have
          s = log(z0+k) * (1+t4)^2 with |t4| <= 2^(-w) */
-      mpfr_mul_2exp (t, z, 1, GMP_RNDN); /* t = 2z * (1+t5) */
+      mpfr_mul_2ui (t, z, 1, GMP_RNDN); /* t = 2z * (1+t5) */
       mpfr_sub_ui (t, t, 1, GMP_RNDN); /* t = 2z-1 * (1+t6)^3 */
       /* since we can write 2z*(1+t5) = (2z-1)*(1+t5') with
          t5' = 2z/(2z-1) * t5, thus |t5'| <= 8/7 * t5 */
       mpfr_mul (s, s, t, GMP_RNDN); /* (2z-1)*log(z) * (1+t7)^6 */
-      mpfr_div_2exp (s, s, 1, GMP_RNDN); /* (z-1/2)*log(z) * (1+t7)^6 */
+      mpfr_div_2ui (s, s, 1, GMP_RNDN); /* (z-1/2)*log(z) * (1+t7)^6 */
       mpfr_sub (s, s, z, GMP_RNDN); /* (z-1/2)*log(z)-z */
       /* s = [(z-1/2)*log(z)-z]*(1+u)^14, s >= 1/2 */
 
@@ -377,7 +455,7 @@ GAMMA_FUNC (mpfr_ptr y, mpfr_srcptr z0, mp_rnd_t rnd)
 
       /* add 1/2*log(2*Pi) and subtract log(z0*(z0+1)*...*(z0+k-1)) */
       mpfr_const_pi (v, GMP_RNDN); /* v = Pi*(1+u) */
-      mpfr_mul_2exp (v, v, 1, GMP_RNDN); /* v = 2*Pi * (1+u) */
+      mpfr_mul_2ui (v, v, 1, GMP_RNDN); /* v = 2*Pi * (1+u) */
       if (k)
         {
           unsigned long l;
@@ -419,7 +497,7 @@ GAMMA_FUNC (mpfr_ptr y, mpfr_srcptr z0, mp_rnd_t rnd)
          thus log(v) = log(v0) + (4k+1)*log(1+u). Since |log(1+u)/u| <= 1.07
          for |u| <= 2^(-3), the absolute error on log(v) is bounded by
          1.07*(4k+1)*u, and the rounding error by ulp(t). */
-      mpfr_div_2exp (t, t, 1, GMP_RNDN);
+      mpfr_div_2ui (t, t, 1, GMP_RNDN);
       /* the error on t is now bounded by ulp(t) + 0.54*(4k+1)*2^(-w).
          We have sqrt(2*Pi)/(z0*(z0+1)*...*(z0+k-1)) <= sqrt(2*Pi)/k! <= 0.5
          since k>=3, thus t <= -0.5 and ulp(t) >= 2^(-w).
@@ -456,3 +534,138 @@ GAMMA_FUNC (mpfr_ptr y, mpfr_srcptr z0, mp_rnd_t rnd)
   MPFR_SAVE_EXPO_FREE (expo);
   return mpfr_check_range (y, inexact, rnd);
 }
+
+#ifndef IS_GAMMA
+
+int
+mpfr_lngamma (mpfr_ptr y, mpfr_srcptr x, mp_rnd_t rnd)
+{
+  int inex;
+
+  MPFR_LOG_FUNC (("x[%#R]=%R rnd=%d", x, x, rnd),
+                 ("lngamma[%#R]=%R inexact=%d", y, y, inex));
+
+  /* special cases */
+  if (MPFR_UNLIKELY (MPFR_IS_SINGULAR (x)))
+    {
+      if (MPFR_IS_NAN (x) || MPFR_IS_NEG (x))
+        {
+          MPFR_SET_NAN (y);
+          MPFR_RET_NAN;
+        }
+      else /* lngamma(+Inf) = lngamma(+0) = +Inf */
+        {
+          MPFR_SET_INF (y);
+          MPFR_SET_POS (y);
+          MPFR_RET (0);  /* exact */
+        }
+    }
+
+  /* if x < 0 and -2k-1 <= x <= -2k, then lngamma(x) = NaN */
+  if (MPFR_IS_NEG (x) && (unit_bit (x) == 0 || mpfr_integer_p (x)))
+    {
+      MPFR_SET_NAN (y);
+      MPFR_RET_NAN;
+    }
+
+  inex = mpfr_lngamma_aux (y, x, rnd);
+  return inex;
+}
+
+int
+mpfr_lgamma (mpfr_ptr y, int *signp, mpfr_srcptr x, mp_rnd_t rnd)
+{
+  int inex;
+
+  MPFR_LOG_FUNC (("x[%#R]=%R rnd=%d", x, x, rnd),
+                 ("lgamma[%#R]=%R inexact=%d", y, y, inex));
+
+  *signp = 1;  /* most common case */
+
+  if (MPFR_UNLIKELY (MPFR_IS_SINGULAR (x)))
+    {
+      if (MPFR_IS_NAN (x))
+        {
+          MPFR_SET_NAN (y);
+          MPFR_RET_NAN;
+        }
+      else
+        {
+          *signp = MPFR_INT_SIGN (x);
+          MPFR_SET_INF (y);
+          MPFR_SET_POS (y);
+          MPFR_RET (0);
+        }
+    }
+
+  if (MPFR_IS_NEG (x))
+    {
+      if (mpfr_integer_p (x))
+        {
+          MPFR_SET_INF (y);
+          MPFR_SET_POS (y);
+          MPFR_RET (0);
+        }
+
+      if (unit_bit (x) == 0)
+        *signp = -1;
+
+      /* For tiny negative x, we have gamma(x) = 1/x - euler + O(x),
+         thus |gamma(x)| = -1/x + euler + O(x), and
+         log |gamma(x)| = -log(-x) - euler*x + O(x^2).
+         More precisely we have for -0.4 <= x < 0:
+         -log(-x) <= log |gamma(x)| <= -log(-x) - x.
+         Since log(x) is not representable, we may have an instance of the
+         Table Maker Dilemma. The only way to ensure correct rounding is to
+         compute an interval [l,h] such that l <= -log(-x) and
+         -log(-x) - x <= h, and check whether l and h round to the same number
+         for the target precision and rounding modes. */
+      if (MPFR_EXP(x) + 1 <= - (mp_exp_t) MPFR_PREC(y))
+        /* since PREC(y) >= 1, this ensures EXP(x) <= -2,
+           thus |x| <= 0.25 < 0.4 */
+        {
+          mpfr_t l, h;
+          int ok, inex2;
+          mp_prec_t w = MPFR_PREC (y) + 14;
+
+          while (1)
+            {
+              mpfr_init2 (l, w);
+              mpfr_init2 (h, w);
+              /* we want a lower bound on -log(-x), thus an upper bound
+                 on log(-x), thus an upper bound on -x. */
+              mpfr_neg (l, x, GMP_RNDU); /* upper bound on -x */
+              mpfr_log (l, l, GMP_RNDU); /* upper bound for log(-x) */
+              mpfr_neg (l, l, GMP_RNDD); /* lower bound for -log(-x) */
+              mpfr_neg (h, x, GMP_RNDD); /* lower bound on -x */
+              mpfr_log (h, h, GMP_RNDD); /* lower bound on log(-x) */
+              mpfr_neg (h, h, GMP_RNDU); /* upper bound for -log(-x) */
+              mpfr_sub (h, h, x, GMP_RNDU); /* upper bound for -log(-x) - x */
+              inex = mpfr_prec_round (l, MPFR_PREC (y), rnd);
+              inex2 = mpfr_prec_round (h, MPFR_PREC (y), rnd);
+              /* Caution: we not only need l = h, but both inexact flags
+                 should agree. Indeed, one of the inexact flags might be
+                 zero. In that case if we assume ln|gamma(x)| cannot be
+                 exact, the other flag should be correct. We are conservative
+                 here and request that both inexact flags agree. */
+              ok = SAME_SIGN (inex, inex2) && mpfr_equal_p (l, h);
+              if (ok)
+                mpfr_set (y, h, rnd); /* exact */
+              mpfr_clear (l);
+              mpfr_clear (h);
+              if (ok)
+                return inex;
+              /* if ulp(log(-x)) <= |x| there is no reason to loop,
+                 since the width of [l, h] will be at least |x| */
+              if (MPFR_EXP(l) < MPFR_EXP(x) + (mp_exp_t) w)
+                break;
+              w += MPFR_INT_CEIL_LOG2(w) + 3;
+            }
+        }
+    }
+
+  inex = mpfr_lngamma_aux (y, x, rnd);
+  return inex;
+}
+
+#endif
