@@ -1,6 +1,7 @@
 /* Utilities for MPFR developers, not exported.
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -22,10 +23,16 @@ MA 02110-1301, USA. */
 #ifndef __MPFR_IMPL_H__
 #define __MPFR_IMPL_H__
 
-/* Include stdio.h iff we are debugging or we want to check */
-#if defined(DEBUG) || defined(WANT_ASSERT)
-# include <stdio.h>
-#endif
+/* Let's include some standard headers unconditionally as they are
+   already needed by several source files or when some options are
+   enabled/disabled, and it is easy to forget them (some configure
+   options may hide the error).
+   Note: If some source file must not have such a header included
+   (which is very unlikely and probably means something broken in
+   this source file), we should do that with some macro (that would
+   also force to disable incompatible features). */
+#include <stdio.h>
+#include <limits.h>
 
 /* Check if we are inside a build of MPFR or inside the test suite.
    This is needed in mpfr.h to export or import the functions.
@@ -164,11 +171,24 @@ MA 02110-1301, USA. */
  ************* Global Internal Variables **************
  ******************************************************/
 
+/* Note: Let's define MPFR_THREAD_ATTR even after a #error to make the
+   error message more visible (e.g. gcc doesn't immediately stop after
+   the #error line and outputs many error messages if MPFR_THREAD_ATTR
+   is not defined). But some compilers will just output a message and
+   may build MPFR "successfully" (without thread support). */
 #ifdef MPFR_USE_THREAD_SAFE
 # if __MPFR_GNUC(3,3) || __MPFR_ICC(8,1,0)
 #  define MPFR_THREAD_ATTR __thread
+# elif defined(_MSC_VER)
+#  if defined(_WINDLL)
+#   error "Can't build MPFR DLL as thread safe."
+#   define MPFR_THREAD_ATTR
+#  else
+#   define MPFR_THREAD_ATTR __declspec( thread )
+#  endif
 # else
-#  error "Can't build MPFR as thread safe"
+#  error "Can't build MPFR as thread safe (you need gcc >= 3.3 or icc >= 8.1)."
+#  define MPFR_THREAD_ATTR
 # endif
 #else
 # define MPFR_THREAD_ATTR
@@ -196,6 +216,9 @@ __MPFR_DECLSPEC extern MPFR_THREAD_ATTR mpfr_cache_t __gmpfr_cache_const_log2;
 __MPFR_DECLSPEC extern MPFR_THREAD_ATTR mpfr_cache_t __gmpfr_cache_const_euler;
 __MPFR_DECLSPEC extern MPFR_THREAD_ATTR mpfr_cache_t __gmpfr_cache_const_catalan;
 
+#define BASE_MAX 36
+__MPFR_DECLSPEC extern const __mpfr_struct __gmpfr_l2b[BASE_MAX-1][2];
+
 __MPFR_DECLSPEC extern const mpfr_t __gmpfr_one;
 __MPFR_DECLSPEC extern const mpfr_t __gmpfr_two;
 __MPFR_DECLSPEC extern const mpfr_t __gmpfr_four;
@@ -280,6 +303,28 @@ __MPFR_DECLSPEC extern const mpfr_t __gmpfr_four;
 #define MPFR_RET_NEVER_GO_HERE()  {MPFR_ASSERTN(0); return 0;}
 
 
+/******************************************************
+ ******************** Warnings ************************
+ ******************************************************/
+
+/* MPFR_WARNING is no longer useful, but let's keep the macro in case
+   it needs to be used again in the future. */
+
+#ifdef MPFR_USE_WARNINGS
+# include <stdlib.h>
+# define MPFR_WARNING(W)                    \
+  do                                        \
+    {                                       \
+      char *q = getenv ("MPFR_QUIET");      \
+      if (q == NULL || *q == 0)             \
+        fprintf (stderr, "MPFR: %s\n", W);  \
+    }                                       \
+  while (0)
+#else
+# define MPFR_WARNING(W)  ((void) 0)
+#endif
+
+
 /******************************************************
  ****************** double macros *********************
  ******************************************************/
@@ -354,8 +399,6 @@ typedef union ieee_double_extract Ieee_double_extract;
 # endif
 #endif
 
-
-
 /******************************************************
  *************** Long double macros *******************
  ******************************************************/
@@ -456,6 +499,15 @@ typedef union {
 
 #endif
 
+/******************************************************
+ *************** _Decimal64 support *******************
+ ******************************************************/
+
+#if MPFR_WANT_DECIMAL_FLOATS
+/* to cast between binary64 and decimal64 */
+union ieee_double_decimal64 { double d; _Decimal64 d64; };
+#endif
+
 /******************************************************
  **************** mpfr_t properties *******************
  ******************************************************/
@@ -481,7 +533,10 @@ typedef union {
  ***************** exponent limits ********************
  ******************************************************/
 
-/* Defined limits and unsigned type of exponent */
+/* Define limits and unsigned type of exponent. The following definitions
+ * depend on mp_exp_t; if this type changes in GMP, these definitions will
+ * need to be modified (alternatively, a mpfr_exp_t type could be defined).
+ */
 #if __GMP_MP_SIZE_T_INT == 1
 typedef unsigned int            mpfr_uexp_t;
 # define MPFR_EXP_MAX (INT_MAX)
@@ -494,12 +549,18 @@ typedef unsigned long int  mpfr_uexp_t;
 #ifndef mp_exp_unsigned_t
 # define mp_exp_unsigned_t mpfr_uexp_t
 #endif
+#define mpfr_get_exp_t(x,r) mpfr_get_si((x),(r))
+#define mpfr_set_exp_t(x,e,r) mpfr_set_si((x),(e),(r))
 
 /* Invalid exponent value (to track bugs...) */
 #define MPFR_EXP_INVALID \
  ((mp_exp_t) 1 << (BITS_PER_MP_LIMB*sizeof(mp_exp_t)/sizeof(mp_limb_t)-2))
 
-/* Definition of the intervals of the exponent limits */
+/* Definition of the exponent limits for MPFR numbers.
+ * These limits are chosen so that if e is such an exponent, then 2e-1 and
+ * 2e+1 are representable. This is useful for intermediate computations,
+ * in particular the multiplication.
+ */
 #undef MPFR_EMIN_MIN
 #undef MPFR_EMIN_MAX
 #undef MPFR_EMAX_MIN
@@ -616,6 +677,9 @@ typedef unsigned long int  mpfr_uexp_t;
 
 #define MPFR_SET_ERANGE() (__gmpfr_flags |= MPFR_FLAGS_ERANGE)
 
+#define SIGN(I) ((I) < 0 ? -1 : (I) > 0)
+#define SAME_SIGN(I1,I2) (SIGN (I1) == SIGN (I2))
+
 
 
 /******************************************************
@@ -783,8 +847,11 @@ extern unsigned char *mpfr_stack;
  *   Undefined if x is 0 */
 #if __MPFR_GNUC(2,95) || __MPFR_ICC(8,1,0)
 # define MPFR_INT_CEIL_LOG2(x)                            \
-    (__extension__ ({int _b; mp_limb_t _limb = (x);       \
-      MPFR_ASSERTN (_limb == (x));                        \
+    (MPFR_UNLIKELY ((x) == 1) ? 0 :                       \
+     __extension__ ({ int _b; mp_limb_t _limb;            \
+      MPFR_ASSERTN ((x) > 1);                             \
+      _limb = (x) - 1;                                    \
+      MPFR_ASSERTN (_limb == (x) - 1);                    \
       count_leading_zeros (_b, _limb);                    \
       (BITS_PER_MP_LIMB - _b); }))
 #else
@@ -853,7 +920,6 @@ do {                                                                  \
 /*   MPFR_TRACE (operation) : execute operation iff DEBUG flag is set */
 /*   MPFR_DUMP (x) : print x (a mpfr_t) on stdout */
 #ifdef DEBUG
-# include <stdio.h>
 # define MPFR_TRACE(x) x
 #else
 # define MPFR_TRACE(x) (void) 0
@@ -1086,37 +1152,74 @@ typedef struct {
  (!MPFR_IS_SINGULAR (b) && mpfr_round_p (MPFR_MANT (b), MPFR_LIMB_SIZE (b),  \
                                          (err), (prec) + ((rnd)==GMP_RNDN)))
 
-/* Assuming that the function as a taylor expansion which looks like:
-    y=o(f(x)) = o(x + g(x)) with |g(x)| <=2^(EXP(x)-err)
-   we can quickly set y to x if x is small (ie err > prec(y)+1) in most
+/* TODO: fix this description (see round_near_x.c). */
+/* Assuming that the function has a Taylor expansion which looks like:
+    y=o(f(x)) = o(x + g(x)) with |g(x)| <= 2^(EXP(v)-err)
+   we can quickly set y to v if x is small (ie err > prec(y)+1) in most
    cases. It assumes that f(x) is not representable exactly as a FP number.
-   x must not be a singular value (NAN, INF or ZERO).
+   v must not be a singular value (NAN, INF or ZERO); usual values are
+   v=1 or v=x.
 
-   y is the destination (a mpfr_t), x the value to set (a mpfr_t),
-   err the error term (a mp_exp_t), dir (an int) is the direction of
-   the commited error (if dir = 0, it rounds towards 0, if dir=1,
-   it rounds away from 0), rnd the rounding mode.
+   y is the destination (a mpfr_t), v the value to set (a mpfr_t),
+   err1+err2 with err2 <= 3 the error term (mp_exp_t's), dir (an int) is
+   the direction of the committed error (if dir = 0, it rounds towards 0,
+   if dir=1, it rounds away from 0), rnd the rounding mode.
 
    It returns from the function a ternary value in case of success.
    If you want to free something, you must fill the "extra" field
    in consequences, otherwise put nothing in it.
 
-   The test is less restrictive thant necessary, but the function
+   The test is less restrictive than necessary, but the function
    will finish the check itself.
+
+   Note: err1 + err2 is allowed to overflow as mp_exp_t, but it must give
+   its real value as mpfr_uexp_t.
 */
-#define MPFR_FAST_COMPUTE_IF_SMALL_INPUT(y,x,err,dir,rnd,extra)     \
-  do {                                                              \
-   mp_exp_t _err = (err);                                           \
-   if (MPFR_UNLIKELY (_err > 0                                      \
-                      && (mpfr_uexp_t) _err > MPFR_PREC (y) + 1))   \
-    {                                                               \
-      int _inexact = mpfr_round_near_x ((y),(x),(err),(dir),(rnd)); \
-      if (_inexact != 0)                                            \
-        {                                                           \
-         extra;                                                     \
-         return _inexact;                                           \
-        }                                                           \
-    }                                                               \
+#define MPFR_FAST_COMPUTE_IF_SMALL_INPUT(y,v,err1,err2,dir,rnd,extra)   \
+  do {                                                                  \
+    mpfr_ptr _y = (y);                                                  \
+    mp_exp_t _err1 = (err1);                                            \
+    mp_exp_t _err2 = (err2);                                            \
+    if (_err1 > 0)                                                      \
+      {                                                                 \
+        mpfr_uexp_t _err = (mpfr_uexp_t) _err1 + _err2;                 \
+        if (MPFR_UNLIKELY (_err > MPFR_PREC (_y) + 1))                  \
+          {                                                             \
+            int _inexact = mpfr_round_near_x (_y,(v),_err,(dir),(rnd)); \
+            if (_inexact != 0)                                          \
+              {                                                         \
+                extra;                                                  \
+                return _inexact;                                        \
+              }                                                         \
+          }                                                             \
+      }                                                                 \
+  } while (0)
+
+/* Variant, to be called somewhere after MPFR_SAVE_EXPO_MARK. This variant
+   is needed when there are some computations before or when some non-zero
+   real constant is used, such as __gmpfr_one for mpfr_cos. */
+#define MPFR_SMALL_INPUT_AFTER_SAVE_EXPO(y,v,err1,err2,dir,rnd,expo,extra) \
+  do {                                                                  \
+    mpfr_ptr _y = (y);                                                  \
+    mp_exp_t _err1 = (err1);                                            \
+    mp_exp_t _err2 = (err2);                                            \
+    if (_err1 > 0)                                                      \
+      {                                                                 \
+        mpfr_uexp_t _err = (mpfr_uexp_t) _err1 + _err2;                 \
+        if (MPFR_UNLIKELY (_err > MPFR_PREC (_y) + 1))                  \
+          {                                                             \
+            int _inexact;                                               \
+            mpfr_clear_flags ();                                        \
+            _inexact = mpfr_round_near_x (_y,(v),_err,(dir),(rnd));     \
+            if (_inexact != 0)                                          \
+              {                                                         \
+                extra;                                                  \
+                MPFR_SAVE_EXPO_UPDATE_FLAGS (expo, __gmpfr_flags);      \
+                MPFR_SAVE_EXPO_FREE (expo);                             \
+                return mpfr_check_range (_y, _inexact, (rnd));          \
+              }                                                         \
+          }                                                             \
+      }                                                                 \
   } while (0)
 
 /******************************************************
@@ -1187,8 +1290,6 @@ typedef struct {
 
 #ifdef MPFR_USE_LOGGING
 
-# include <stdio.h>
-
 /* Check if we can support this feature */
 # ifdef MPFR_USE_THREAD_SAFE
 #  error "Enable either `Logging' or `thread-safe', not both"
@@ -1282,26 +1383,31 @@ struct mpfr_group_t {
 #define MPFR_GROUP_DECL(g) struct mpfr_group_t g
 #define MPFR_GROUP_CLEAR(g) do {                                 \
  if (MPFR_UNLIKELY ((g).alloc != 0)) {                           \
-    MPFR_ASSERTD ((g).mant != (g).tab);                          \
-    (*__gmp_free_func)((g).mant, (g).alloc);                     \
+   MPFR_ASSERTD ((g).mant != (g).tab);                           \
+   (*__gmp_free_func) ((g).mant, (g).alloc);                     \
  }} while (0)
 
-#define MPFR_GROUP_INIT_TEMPLATE(g, prec, num, handler) do {     \
- mp_prec_t _prec = (prec);                                       \
- mp_size_t _size;                                                \
- MPFR_ASSERTD (_prec >= MPFR_PREC_MIN);                          \
- if (MPFR_UNLIKELY (_prec > MPFR_PREC_MAX)) mpfr_abort_prec_max(); \
- _size = (mp_prec_t)(_prec +BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB;\
- if (MPFR_UNLIKELY (_size*(num) > MPFR_GROUP_STATIC_SIZE)) {     \
-  (g).alloc = (num)*_size*sizeof (mp_limb_t);                    \
-  (g).mant = (*__gmp_allocate_func) ((g).alloc);                 \
- } else {                                                        \
-  (g).alloc = 0;                                                 \
-  (g).mant = (g).tab;                                            \
- }                                                               \
- handler;                                                        \
+#define MPFR_GROUP_INIT_TEMPLATE(g, prec, num, handler) do {            \
+ mp_prec_t _prec = (prec);                                              \
+ mp_size_t _size;                                                       \
+ MPFR_ASSERTD (_prec >= MPFR_PREC_MIN);                                 \
+ if (MPFR_UNLIKELY (_prec > MPFR_PREC_MAX))                             \
+   mpfr_abort_prec_max ();                                              \
+ _size = (mp_prec_t) (_prec + BITS_PER_MP_LIMB - 1) / BITS_PER_MP_LIMB; \
+ if (MPFR_UNLIKELY (_size * (num) > MPFR_GROUP_STATIC_SIZE))            \
+   {                                                                    \
+     (g).alloc = (num) * _size * sizeof (mp_limb_t);                    \
+     (g).mant = (mp_limb_t *) (*__gmp_allocate_func) ((g).alloc);       \
+   }                                                                    \
+ else                                                                   \
+   {                                                                    \
+     (g).alloc = 0;                                                     \
+     (g).mant = (g).tab;                                                \
+   }                                                                    \
+ handler;                                                               \
  } while (0)
-#define MPFR_GROUP_TINIT(g, n, x) MPFR_TMP_INIT1 ((g).mant+_size*(n), x, _prec)
+#define MPFR_GROUP_TINIT(g, n, x)                       \
+  MPFR_TMP_INIT1 ((g).mant + _size * (n), x, _prec)
 
 #define MPFR_GROUP_INIT_1(g, prec, x)                            \
  MPFR_GROUP_INIT_TEMPLATE(g, prec, 1, MPFR_GROUP_TINIT(g, 0, x))
@@ -1327,21 +1433,22 @@ struct mpfr_group_t {
    MPFR_GROUP_TINIT(g, 2, z);MPFR_GROUP_TINIT(g, 3, t);          \
    MPFR_GROUP_TINIT(g, 4, a);MPFR_GROUP_TINIT(g, 5, b))
 
-#define MPFR_GROUP_REPREC_TEMPLATE(g, prec, num, handler) do {   \
- mp_prec_t _prec = (prec);                                       \
- size_t    _oalloc = (g).alloc;                                  \
- mp_size_t _size;                                                \
- MPFR_ASSERTD (_prec >= MPFR_PREC_MIN);                          \
- if (MPFR_UNLIKELY (_prec > MPFR_PREC_MAX)) mpfr_abort_prec_max(); \
- _size = (mp_prec_t)(_prec +BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB;\
- (g).alloc = (num)*_size*sizeof(mp_limb_t);                      \
- if (MPFR_LIKELY (_oalloc == 0))                                 \
-  (g).mant = (*__gmp_allocate_func) ((g).alloc);                 \
- else                                                            \
-  (g).mant = (*__gmp_reallocate_func)((g).mant,_oalloc,(g).alloc);\
- handler;                                                        \
-} while (0)
-
+#define MPFR_GROUP_REPREC_TEMPLATE(g, prec, num, handler) do {          \
+ mp_prec_t _prec = (prec);                                              \
+ size_t    _oalloc = (g).alloc;                                         \
+ mp_size_t _size;                                                       \
+ MPFR_ASSERTD (_prec >= MPFR_PREC_MIN);                                 \
+ if (MPFR_UNLIKELY (_prec > MPFR_PREC_MAX))                             \
+   mpfr_abort_prec_max ();                                              \
+ _size = (mp_prec_t) (_prec + BITS_PER_MP_LIMB - 1) / BITS_PER_MP_LIMB; \
+ (g).alloc = (num) * _size * sizeof (mp_limb_t);                        \
+ if (MPFR_LIKELY (_oalloc == 0))                                        \
+   (g).mant = (mp_limb_t *) (*__gmp_allocate_func) ((g).alloc);         \
+ else                                                                   \
+   (g).mant = (mp_limb_t *)                                             \
+     (*__gmp_reallocate_func) ((g).mant, _oalloc, (g).alloc);           \
+ handler;                                                               \
+ } while (0)
 
 #define MPFR_GROUP_REPREC_1(g, prec, x)                          \
  MPFR_GROUP_REPREC_TEMPLATE(g, prec, 1, MPFR_GROUP_TINIT(g, 0, x))
@@ -1453,6 +1560,7 @@ __MPFR_DECLSPEC int  mpfr_cache _MPFR_PROTO ((mpfr_ptr, mpfr_cache_t,
 
 __MPFR_DECLSPEC void mpfr_mulhigh_n _MPFR_PROTO ((mp_ptr, mp_srcptr,
                                                   mp_srcptr, mp_size_t));
+__MPFR_DECLSPEC void mpfr_sqrhigh_n _MPFR_PROTO ((mp_ptr, mp_srcptr, mp_size_t));
 
 __MPFR_DECLSPEC int mpfr_round_p _MPFR_PROTO ((mp_limb_t *, mp_size_t,
                                                mp_exp_t, mp_prec_t));
@@ -1462,7 +1570,8 @@ __MPFR_DECLSPEC void mpfr_dump_mant _MPFR_PROTO ((const mp_limb_t *,
                                                   mp_prec_t));
 
 __MPFR_DECLSPEC int mpfr_round_near_x _MPFR_PROTO ((mpfr_ptr, mpfr_srcptr,
-                                                    mp_exp_t, int, mp_rnd_t));
+                                                    mpfr_uexp_t, int,
+                                                    mp_rnd_t));
 __MPFR_DECLSPEC void mpfr_abort_prec_max _MPFR_PROTO ((void))
        MPFR_NORETURN_ATTR;
 
