@@ -1,6 +1,7 @@
 /* mpfr_set_default_prec, mpfr_get_default_prec -- set/get default precision
 
-Copyright 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,21 +17,23 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
 
 /* default is IEEE double precision, i.e. 53 bits */
-mp_prec_t __gmpfr_default_fp_bit_precision = IEEE_DBL_MANT_DIG;
+mp_prec_t MPFR_THREAD_ATTR __gmpfr_default_fp_bit_precision \
+  = IEEE_DBL_MANT_DIG;
 
 void
 mpfr_set_default_prec (mp_prec_t prec)
 {
-  MPFR_ASSERTN(prec >= MPFR_PREC_MIN && prec <= MPFR_PREC_MAX);
+  MPFR_ASSERTN (prec >= MPFR_PREC_MIN && prec <= MPFR_PREC_MAX);
   __gmpfr_default_fp_bit_precision = prec;
 }
 
+#undef mpfr_get_default_prec
 mp_prec_t
 mpfr_get_default_prec (void)
 {
