@@ -1,6 +1,7 @@
 /* Test file for in-place operations.
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation.
+Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include <stdio.h>
@@ -526,81 +527,84 @@ main (void)
   tests_start_mpfr ();
 
   p = (randlimb () % 200)+ MPFR_PREC_MIN;
-  for (rnd = GMP_RNDN ; rnd < GMP_RND_MAX ; (mp_rnd_t) ((int) rnd++)) {
-    test3 (mpfr_add, "mpfr_add", p, rnd);
-    test3 (mpfr_sub, "mpfr_sub", p, rnd);
-    test3 (mpfr_mul, "mpfr_mul", p, rnd);
-    test3 (mpfr_div, "mpfr_div", p, rnd);
+  RND_LOOP (rnd)
+    {
+      test3 (mpfr_add, "mpfr_add", p, rnd);
+      test3 (mpfr_sub, "mpfr_sub", p, rnd);
+      test3 (mpfr_mul, "mpfr_mul", p, rnd);
+      test3 (mpfr_div, "mpfr_div", p, rnd);
 
-    test3 (mpfr_agm, "mpfr_agm", p, rnd);
-    test3 (mpfr_min, "mpfr_min", p, rnd);
-    test3 (mpfr_max, "mpfr_max", p, rnd);
-    test3 (reldiff_wrapper, "mpfr_reldiff", p, rnd);
+      test3 (mpfr_agm, "mpfr_agm", p, rnd);
+      test3 (mpfr_min, "mpfr_min", p, rnd);
+      test3 (mpfr_max, "mpfr_max", p, rnd);
+      test3 (reldiff_wrapper, "mpfr_reldiff", p, rnd);
 
-    test2a (mpfr_round, "mpfr_round", p);
-    test2a (mpfr_ceil, "mpfr_ceil", p);
-    test2a (mpfr_floor, "mpfr_floor", p);
-    test2a (mpfr_trunc, "mpfr_trunc", p);
+      test2a (mpfr_round, "mpfr_round", p);
+      test2a (mpfr_ceil, "mpfr_ceil", p);
+      test2a (mpfr_floor, "mpfr_floor", p);
+      test2a (mpfr_trunc, "mpfr_trunc", p);
 
-    test2ui (mpfr_add_ui, "mpfr_add_ui", p, rnd);
-    test2ui (mpfr_mul_2exp, "mpfr_mul_2exp", p, rnd);
-    test2ui (mpfr_mul_ui, "mpfr_mul_ui", p, rnd);
-    test2ui (mpfr_pow_ui, "mpfr_pow_ui", p, rnd);
-    test2ui (mpfr_sub_ui, "mpfr_sub_ui", p, rnd);
-    test2ui (mpfr_div_2exp, "mpfr_div_2exp", p, rnd);
-    test2ui (mpfr_div_ui, "mpfr_div_ui", p, rnd);
+      test2ui (mpfr_add_ui, "mpfr_add_ui", p, rnd);
+      test2ui (mpfr_mul_2exp, "mpfr_mul_2exp", p, rnd);
+      test2ui (mpfr_mul_ui, "mpfr_mul_ui", p, rnd);
+      test2ui (mpfr_pow_ui, "mpfr_pow_ui", p, rnd);
+      test2ui (mpfr_sub_ui, "mpfr_sub_ui", p, rnd);
+      test2ui (mpfr_div_2exp, "mpfr_div_2exp", p, rnd);
+      test2ui (mpfr_div_ui, "mpfr_div_ui", p, rnd);
 
-    testui2 (mpfr_ui_div, "mpfr_ui_div", p, rnd);
-    testui2 (mpfr_ui_sub, "mpfr_ui_sub", p, rnd);
-    testui2 (mpfr_ui_pow, "mpfr_ui_pow", p, rnd);
+      testui2 (mpfr_ui_div, "mpfr_ui_div", p, rnd);
+      testui2 (mpfr_ui_sub, "mpfr_ui_sub", p, rnd);
+      testui2 (mpfr_ui_pow, "mpfr_ui_pow", p, rnd);
 
-    test2 (mpfr_sqr, "mpfr_sqr", p, rnd);
-    test2 (mpfr_sqrt, "mpfr_sqrt", p, rnd);
-    test2 (mpfr_abs, "mpfr_abs", p, rnd);
-    test2 (mpfr_neg, "mpfr_neg", p, rnd);
+      test2 (mpfr_sqr, "mpfr_sqr", p, rnd);
+      test2 (mpfr_sqrt, "mpfr_sqrt", p, rnd);
+      test2 (mpfr_abs, "mpfr_abs", p, rnd);
+      test2 (mpfr_neg, "mpfr_neg", p, rnd);
 
-    test2 (mpfr_log, "mpfr_log", p, rnd);
-    test2 (mpfr_log2, "mpfr_log2", p, rnd);
-    test2 (mpfr_log10, "mpfr_log10", p, rnd);
-    test2 (mpfr_log1p, "mpfr_log1p", p, rnd);
+      test2 (mpfr_log, "mpfr_log", p, rnd);
+      test2 (mpfr_log2, "mpfr_log2", p, rnd);
+      test2 (mpfr_log10, "mpfr_log10", p, rnd);
+      test2 (mpfr_log1p, "mpfr_log1p", p, rnd);
 
-    test2 (mpfr_exp, "mpfr_exp", p, rnd);
-    test2 (mpfr_exp2, "mpfr_exp2", p, rnd);
-    test2 (mpfr_exp10, "mpfr_exp10", p, rnd);
-    test2 (mpfr_expm1, "mpfr_expm1", p, rnd);
-    test2 (mpfr_eint, "mpfr_eint", p, rnd);
+      test2 (mpfr_exp, "mpfr_exp", p, rnd);
+      test2 (mpfr_exp2, "mpfr_exp2", p, rnd);
+      test2 (mpfr_exp10, "mpfr_exp10", p, rnd);
+      test2 (mpfr_expm1, "mpfr_expm1", p, rnd);
+      test2 (mpfr_eint, "mpfr_eint", p, rnd);
 
-    test2 (mpfr_sinh, "mpfr_sinh", p, rnd);
-    test2 (mpfr_cosh, "mpfr_cosh", p, rnd);
-    test2 (mpfr_tanh, "mpfr_tanh", p, rnd);
-    test2 (mpfr_asinh, "mpfr_asinh", p, rnd);
-    test2 (mpfr_acosh, "mpfr_acosh", p, rnd);
-    test2 (mpfr_atanh, "mpfr_atanh", p, rnd);
+      test2 (mpfr_sinh, "mpfr_sinh", p, rnd);
+      test2 (mpfr_cosh, "mpfr_cosh", p, rnd);
+      test2 (mpfr_tanh, "mpfr_tanh", p, rnd);
+      test2 (mpfr_asinh, "mpfr_asinh", p, rnd);
+      test2 (mpfr_acosh, "mpfr_acosh", p, rnd);
+      test2 (mpfr_atanh, "mpfr_atanh", p, rnd);
 
-    test2 (mpfr_asin, "mpfr_asin", p, rnd);
-    test2 (mpfr_acos, "mpfr_acos", p, rnd);
-    test2 (mpfr_atan, "mpfr_atan", p, rnd);
-    test2 (mpfr_cos, "mpfr_cos", p, rnd);
-    test2 (mpfr_sin, "mpfr_sin", p, rnd);
-    test2 (mpfr_tan, "mpfr_tan", p, rnd);
+      test2 (mpfr_asin, "mpfr_asin", p, rnd);
+      test2 (mpfr_acos, "mpfr_acos", p, rnd);
+      test2 (mpfr_atan, "mpfr_atan", p, rnd);
+      test2 (mpfr_cos, "mpfr_cos", p, rnd);
+      test2 (mpfr_sin, "mpfr_sin", p, rnd);
+      test2 (mpfr_tan, "mpfr_tan", p, rnd);
 
-    test2 (mpfr_sec, "mpfr_sec", p, rnd);
-    test2 (mpfr_csc, "mpfr_csc", p, rnd);
-    test2 (mpfr_cot, "mpfr_cot", p, rnd);
+      test2 (mpfr_sec, "mpfr_sec", p, rnd);
+      test2 (mpfr_csc, "mpfr_csc", p, rnd);
+      test2 (mpfr_cot, "mpfr_cot", p, rnd);
 
-    test2 (mpfr_erf, "mpfr_erf", p, rnd);
-    test2 (mpfr_erfc, "mpfr_erfc", p, rnd);
+      test2 (mpfr_erf,  "mpfr_erf",  p, rnd);
+      test2 (mpfr_erfc, "mpfr_erfc", p, rnd);
+      test2 (mpfr_j0,   "mpfr_j0",   p, rnd);
+      test2 (mpfr_j1,   "mpfr_j1",   p, rnd);
 
-    test2 (mpfr_zeta, "mpfr_zeta", p, rnd);
-    test2 (mpfr_gamma, "mpfr_gamma", p, rnd);
-    test2 (mpfr_rint, "mpfr_rint", p, rnd);
-    test2 (mpfr_frac, "mpfr_frac", p, rnd);
+      test2 (mpfr_zeta, "mpfr_zeta", p, rnd);
+      test2 (mpfr_gamma, "mpfr_gamma", p, rnd);
+      test2 (mpfr_rint, "mpfr_rint", p, rnd);
+      test2 (mpfr_frac, "mpfr_frac", p, rnd);
 
-    test3 (mpfr_pow, "mpfr_pow", p, rnd);
-    test3 (mpfr_atan2, "mpfr_atan2", p, rnd);
+      test3 (mpfr_pow, "mpfr_pow", p, rnd);
+      test3 (mpfr_atan2, "mpfr_atan2", p, rnd);
 
-    test4 (mpfr_fma, "mpfr_fma", p, rnd);
-  }
+      test4 (mpfr_fma, "mpfr_fma", p, rnd);
+    }
 
   tests_end_mpfr ();
   return 0;
