@@ -1,6 +1,7 @@
 /* Test file for mpfr_abs.
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,8 +17,8 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -43,12 +44,12 @@ check_inexact (void)
       mpfr_set_prec (absx, p);
       mpfr_random (x);
       if (randlimb () % 2)
-	{
-	  mpfr_set (absx, x, GMP_RNDN);
-	  mpfr_neg (x, x, GMP_RNDN);
-	}
+        {
+          mpfr_set (absx, x, GMP_RNDN);
+          mpfr_neg (x, x, GMP_RNDN);
+        }
       else
-	mpfr_set (absx, x, GMP_RNDN);
+        mpfr_set (absx, x, GMP_RNDN);
       for (q=2; q<2*p; q++)
         {
           mpfr_set_prec (y, q);
@@ -82,10 +83,10 @@ check_cmp(int argc, char *argv[])
   mpfr_t x, y;
   int n, k, rnd;
 
-  mpfr_inits2(53, x, y, NULL);
+  mpfr_inits2 (53, x, y, (void *) 0);
 
   mpfr_set_ui(x, 1, GMP_RNDN);
-  mpfr_abs(x, x, GMP_RNDN);
+  (mpfr_abs) (x, x, GMP_RNDN);
   if (mpfr_cmp_ui (x, 1))
     {
       printf ("Error in mpfr_abs(1.0)\n");
@@ -142,16 +143,16 @@ check_cmp(int argc, char *argv[])
       MPFR_SET_POS(x);
       if (mpfr_cmp(x,y))
         {
-          printf ("Mismatch for sign=%d and x=", sign); 
-	  mpfr_print_binary(x);
+          printf ("Mismatch for sign=%d and x=", sign);
+          mpfr_print_binary(x);
           printf ("\nResults=");
-          mpfr_print_binary(y); 
-	  putchar ('\n');
+          mpfr_print_binary(y);
+          putchar ('\n');
           exit (1);
         }
     }
 
-  mpfr_clears(x, y, NULL);
+  mpfr_clears (x, y, (void *) 0);
 }
 
 int
