@@ -1,6 +1,7 @@
 /* Test file for mpfr_add1sp.
 
-Copyright 2004, 2005 Free Software Foundation.
+Copyright 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,8 +17,8 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,7 +36,7 @@ check_overflow (void)
   set_emin (-1021);
   set_emax (1024);
 
-  mpfr_inits (x, y, z1, z2, NULL);
+  mpfr_inits (x, y, z1, z2, (void *) 0);
 
   mpfr_set_str1 (x, "8.00468257869324898448e+307");
   mpfr_set_str1 (y, "7.44784712422708645156e+307");
@@ -46,7 +47,7 @@ check_overflow (void)
       printf ("Overflow bug in add1sp.\n");
       exit (1);
     }
-  mpfr_clears (x, y, z1, z2, NULL);
+  mpfr_clears (x, y, z1, z2, (void *) 0);
 }
 
 int main(void)
@@ -102,7 +103,7 @@ void check_random(mp_prec_t p)
   int r;
   int i, inexact1, inexact2;
 
-  mpfr_inits2(p, a1,b,c,a2, NULL);
+  mpfr_inits2 (p, a1, b, c, a2, (void *) 0);
 
   for (i = 0 ; i < 500 ; i++)
     {
@@ -125,7 +126,7 @@ void check_random(mp_prec_t p)
         }
     }
 
-  mpfr_clears(a1,a2,b,c,NULL);
+  mpfr_clears (a1, a2, b, c, (void *) 0);
 }
 
 void check_special(void)
@@ -135,7 +136,7 @@ void check_special(void)
   mpfr_prec_t p;
   int i = -1, inexact1, inexact2;
 
-  mpfr_inits(a1,a2,b,c,NULL);
+  mpfr_inits (a1, a2, b, c, (void *) 0);
 
   for (r = 0 ; r < GMP_RND_MAX ; r++)
     {
@@ -158,5 +159,18 @@ void check_special(void)
         STD_ERROR2;
     }
 
-  mpfr_clears(a1,a2,b,c,NULL);
+  mpfr_set_prec (c, 2);
+  mpfr_set_prec (a1, 2);
+  mpfr_set_prec (a2, 2);
+  mpfr_set_str_binary (c, "1.0e1");
+  mpfr_set_str_binary (a2, "1.1e-1");
+  mpfr_set_str_binary (a1, "0.11E2");
+  mpfr_add1sp (a2, c, a2, GMP_RNDN);
+  if (mpfr_cmp (a1, a2))
+    {
+      printf ("Regression reuse test failed!\n");
+      exit (1);
+    }
+
+  mpfr_clears (a1, a2, b, c, (void *) 0);
 }
