@@ -1,6 +1,7 @@
 /* Test file for mpfr_agm.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -31,7 +32,7 @@ check4 (const char *as, const char *bs, mp_rnd_t rnd_mode, const char *res)
 {
   mpfr_t ta, tb, tres;
 
-  mpfr_inits2(53, ta, tb, tres, NULL);
+  mpfr_inits2 (53, ta, tb, tres, (void *) 0);
 
   mpfr_set_str1 (ta, as);
   mpfr_set_str1 (tb, bs);
@@ -46,7 +47,7 @@ check4 (const char *as, const char *bs, mp_rnd_t rnd_mode, const char *res)
       putchar('\n');
       exit (1);
   }
-  mpfr_clears (ta, tb, tres, NULL);
+  mpfr_clears (ta, tb, tres, (void *) 0);
 }
 
 static void
