@@ -1,6 +1,7 @@
 /* Test file for exceptions.
 
-Copyright 2001, 2002, 2003, 2004, 2005 Free Software Foundation.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,15 +17,29 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
 
 #include "mpfr-test.h"
 
-#define ERROR(s) do { printf(s); exit(1); } while(0)
+#define ERROR(s) do { printf(s"\n"); exit(1); } while(0)
+
+/* Test powerof2 */
+static void
+check_powerof2 (void)
+{
+  mpfr_t x;
+
+  mpfr_init (x);
+  mpfr_set_ui (x, 1, GMP_RNDN);
+  MPFR_ASSERTN (mpfr_powerof2_raw (x));
+  mpfr_set_ui (x, 3, GMP_RNDN);
+  MPFR_ASSERTN (!mpfr_powerof2_raw (x));
+  mpfr_clear (x);
+}
 
 /* Test default rounding mode */
 static void
@@ -35,16 +50,16 @@ check_default_rnd (void)
   for(r = 0 ; r < GMP_RND_MAX ; r++)
     {
       mpfr_set_default_rounding_mode ((mp_rnd_t) r);
-      t = mpfr_get_default_rounding_mode();
+      t = (mpfr_get_default_rounding_mode) ();
       if ((mp_rnd_t) r != t)
-	ERROR("ERROR in setting / getting default rounding mode (1)\n");
+        ERROR("ERROR in setting / getting default rounding mode (1)");
     }
   mpfr_set_default_rounding_mode ((mp_rnd_t) 4);
   if (mpfr_get_default_rounding_mode() != GMP_RNDD)
-    ERROR("ERROR in setting / getting default rounding mode (2)\n");
-  mpfr_set_default_rounding_mode ((mp_rnd_t) -1);
+    ERROR("ERROR in setting / getting default rounding mode (2)");
+  mpfr_set_default_rounding_mode((mp_rnd_t) -1);
   if (mpfr_get_default_rounding_mode() != GMP_RNDD)
-    ERROR("ERROR in setting / getting default rounding mode (3)\n");
+    ERROR("ERROR in setting / getting default rounding mode (3)");
 }
 
 static void
@@ -63,13 +78,38 @@ check_emin_emax (void)
   if ((mpfr_get_emax)() != MPFR_EMAX_MAX)
     ERROR("get_emax FAILED!");
   if ((mpfr_set_emax)(MPFR_EMAX_MAX+1) == 0)
-    ERROR("set_emax failed! (2)");  
+    ERROR("set_emax failed! (2)");
+
+  if ((mpfr_get_emin_min) () != MPFR_EMIN_MIN)
+    ERROR ("get_emin_min");
+  if ((mpfr_get_emin_max) () != MPFR_EMIN_MAX)
+    ERROR ("get_emin_max");
+  if ((mpfr_get_emax_min) () != MPFR_EMAX_MIN)
+    ERROR ("get_emax_min");
+  if ((mpfr_get_emax_max) () != MPFR_EMAX_MAX)
+    ERROR ("get_emax_max");
+}
+
+static void
+check_set_get_prec (void)
+{
+  mpfr_t x;
+
+  mpfr_init2 (x, 17);
+  if (mpfr_get_prec (x) != 17 || (mpfr_get_prec)(x) != 17)
+    ERROR ("mpfr_get_prec");
+  mpfr_clear (x);
 }
 
 static void
 mpfr_set_double_range (void)
 {
+  mpfr_set_default_prec (54);
+  if (mpfr_get_default_prec () != 54)
+    ERROR ("get_default_prec failed (1)");
   mpfr_set_default_prec (53);
+  if ((mpfr_get_default_prec) () != 53)
+    ERROR ("get_default_prec failed (2)");
 
   /* in double precision format, the unbiased exponent is between 0 and
      2047, where 0 is used for subnormal numbers, and 2047 for special
@@ -123,7 +163,7 @@ check_flags (void)
 
   (mpfr_clear_erangeflag) ();
   mpfr_set_ui (x, 1, GMP_RNDN);
-  mpfr_mul_2exp (x, x, 1024, GMP_RNDN); 
+  mpfr_mul_2exp (x, x, 1024, GMP_RNDN);
   mpfr_get_ui (x, GMP_RNDN);
   if (!(mpfr_erangeflag_p)())
     ERROR ("ERROR: No erange flag!\n");
@@ -155,7 +195,7 @@ test_set_underflow (void)
           int inex;
 
           j = s < 0 && i > 1 ? 5 - i : i;
-          inex = mpfr_set_underflow (x, (mp_rnd_t) i, s);
+          inex = mpfr_underflow (x, (mp_rnd_t) i, s);
           if (mpfr_cmp (x, r[j]) || inex * t[j] <= 0)
             {
               printf ("Error in test_set_underflow, sign = %d,"
@@ -198,7 +238,7 @@ test_set_overflow (void)
           int inex;
 
           j = s < 0 && i > 1 ? 5 - i : i;
-          inex = mpfr_set_overflow (x, (mp_rnd_t) i, s);
+          inex = mpfr_overflow (x, (mp_rnd_t) i, s);
           if (mpfr_cmp (x, r[j]) || inex * t[j] <= 0)
             {
               printf ("Error in test_set_overflow, sign = %d,"
@@ -217,6 +257,24 @@ test_set_overflow (void)
   mpfr_clears (x, inf, max, (mpfr_ptr) 0);
 }
 
+static void
+check_set () {
+  mpfr_clear_flags ();
+
+  mpfr_set_overflow ();
+  MPFR_ASSERTN ((mpfr_overflow_p) ());
+  mpfr_set_underflow ();
+  MPFR_ASSERTN ((mpfr_underflow_p) ());
+  mpfr_set_nanflag ();
+  MPFR_ASSERTN ((mpfr_nanflag_p) ());
+  mpfr_set_inexflag ();
+  MPFR_ASSERTN ((mpfr_inexflag_p) ());
+  mpfr_set_erangeflag ();
+  MPFR_ASSERTN ((mpfr_erangeflag_p) ());
+
+  mpfr_clear_flags ();
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -312,6 +370,9 @@ main (int argc, char *argv[])
 
   check_emin_emax();
   check_flags();
+  check_set_get_prec ();
+  check_powerof2 ();
+  check_set ();
 
   tests_end_mpfr ();
   return 0;
