@@ -1,6 +1,7 @@
 /* Test file for mpfr_set_str.
 
-Copyright 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,8 +17,8 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,7 +45,7 @@ MA 02111-1307, USA. */
       exit (1); \
     }
 
-static void 
+static void
 check_underflow (void)
 {
   mpfr_t a;
@@ -60,8 +61,8 @@ check_underflow (void)
   if (!MPFR_IS_ZERO (a))
     {
       printf("ERROR for mpfr_set_str (a, \"0.00000000001\", 10, GMP_RNDN)\n"
-	     " with emin=-20\n"
-	     "res=%d\n", res);
+             " with emin=-20\n"
+             "res=%d\n", res);
       mpfr_dump (a);
       exit (1);
     }
@@ -92,11 +93,12 @@ main (int argc, char *argv[])
 
   tests_start_mpfr ();
 
-  if (argc>=2) /* tset_str <string> <prec> */
+  if (argc >= 2) /* tset_str <string> [<prec>] [<base>] */
     {
-      prec = (argc>=3) ? atoi(argv[2]) : 53;
+      prec = (argc >= 3) ? atoi (argv[2]) : 53;
+      base = (argc >= 4) ? atoi (argv[3]) : 2;
       mpfr_init2 (x, prec);
-      mpfr_set_str_binary (x, argv[1]);
+      mpfr_set_str (x, argv[1], base, GMP_RNDN);
       mpfr_out_str (stdout, 10, 0, x, GMP_RNDN);
       puts ("");
       mpfr_clear (x);
@@ -111,12 +113,12 @@ main (int argc, char *argv[])
 
   bd = randlimb () & 8;
 
-  str2 = str = (*__gmp_allocate_func) (nc * sizeof(char));
+  str2 = str = (char*) (*__gmp_allocate_func) (nc * sizeof(char));
 
   if (bd)
     {
       for(k = 1; k <= bd; k++)
-	*(str2++) = (randlimb () & 1) + '0';
+        *(str2++) = (randlimb () & 1) + '0';
     }
   else
     *(str2++) = '0';
@@ -204,20 +206,20 @@ main (int argc, char *argv[])
       logbase = (randlimb () % 5) + 1;
       base = 1 << logbase;
       /* Warning: the number of bits needed to print exactly a number of
-	 'prec' bits in base 2^logbase may be greater than ceil(prec/logbase),
-	 for example 0.11E-1 in base 2 cannot be written exactly with only
-	 one digit in base 4 */
+         'prec' bits in base 2^logbase may be greater than ceil(prec/logbase),
+         for example 0.11E-1 in base 2 cannot be written exactly with only
+         one digit in base 4 */
       if (base == 2)
-	baseprec = prec;
+        baseprec = prec;
       else
-	baseprec = 1 + (prec - 2 + logbase) / logbase;
-      str = mpfr_get_str (NULL, &e, base, baseprec, x, k);
-      mpfr_set_str (y, str, base, k);
+        baseprec = 1 + (prec - 2 + logbase) / logbase;
+      str = mpfr_get_str (NULL, &e, base, baseprec, x, (mp_rnd_t) k);
+      mpfr_set_str (y, str, base, (mp_rnd_t) k);
       MPFR_EXP(y) += logbase * (e - strlen (str));
       if (mpfr_cmp (x, y))
         {
           printf ("mpfr_set_str o mpfr_get_str <> id for rnd_mode=%s\n",
-                  mpfr_print_rnd_mode (k));
+                  mpfr_print_rnd_mode ((mp_rnd_t) k));
           printf ("x=");
           mpfr_print_binary (x);
           puts ("");
@@ -656,7 +658,7 @@ main (int argc, char *argv[])
   CHECK53(y, "5783893e-128", GMP_RNDU, x,
           "0.10011000111100000110011110000101100111110011101110101E-402",
           38);
-  
+
   CHECK53(y, "87575437e-310", GMP_RNDD, x,
           "0.11000000001011100000110011110011010000000010001010110E-1003",
           39);
@@ -743,7 +745,7 @@ main (int argc, char *argv[])
     mpfr_set_prec (x, mp_bits_per_limb); /* x and y have only one limb */
     mpfr_set_prec (y, mp_bits_per_limb);
 
-    str = (*__gmp_allocate_func) (N + 20);
+    str = (char*) (*__gmp_allocate_func) (N + 20);
 
     mpfr_set_ui (x, 1, GMP_RNDN); /* ensures that x is not NaN or Inf */
     for (; nb_digit < N; nb_digit *= 10)
@@ -780,7 +782,7 @@ main (int argc, char *argv[])
                   printf ("return    : ");
                   mpfr_print_binary (y);
                   puts ("");
-		  exit (1);
+                  exit (1);
                 }
             }
 
@@ -827,7 +829,7 @@ main (int argc, char *argv[])
   mpfr_set_prec (y, 135);
   mpfr_set_str (y, "23833565676460972739462619524519814462546", 10, GMP_RNDN);
   MPFR_ASSERTN (mpfr_cmp (x, y) == 0 && ret == 0);
-  
+
   /* coverage test for set_str_binary */
   mpfr_set_str_binary (x, "NaN");
   MPFR_ASSERTN(mpfr_nan_p (x));
