@@ -1,6 +1,7 @@
 /* tsum -- test file for the list summation function
 
-Copyright 2004, 2005 Free Software Foundation.
+Copyright 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include <stdlib.h>
@@ -150,7 +151,7 @@ test_sum (mp_prec_t f, unsigned long n)
   tab = (mpfr_t *) malloc (n * sizeof(mpfr_t));
   for (i = 0; i < n; i++)
     mpfr_init2 (tab[i], f);
-  mpfr_inits2 (f, sum, real_sum, real_non_rounded, NULL);
+  mpfr_inits2 (f, sum, real_sum, real_non_rounded, (void *) 0);
 
   /* First Uniform */
   for (i = 0; i < n; i++)
@@ -192,7 +193,7 @@ test_sum (mp_prec_t f, unsigned long n)
   /* Clear stuff */
   for (i = 0; i < n; i++)
     mpfr_clear (tab[i]);
-  mpfr_clears (sum, real_sum, real_non_rounded, NULL);
+  mpfr_clears (sum, real_sum, real_non_rounded, (void *) 0);
   free (tab);
 }
 
@@ -203,7 +204,7 @@ void check_special (void)
   mpfr_ptr tabp[3];
   int i;
 
-  mpfr_inits (tab[0], tab[1], tab[2], r, NULL);
+  mpfr_inits (tab[0], tab[1], tab[2], r, (void *) 0);
   tabp[0] = tab[0];
   tabp[1] = tab[1];
   tabp[2] = tab[2];
@@ -276,7 +277,7 @@ void check_special (void)
       exit (1);
     }
 
-  mpfr_clears (tab[0], tab[1], tab[2], r, NULL);
+  mpfr_clears (tab[0], tab[1], tab[2], r, (void *) 0);
 }
 
 
@@ -290,8 +291,8 @@ main (void)
 
   check_special ();
   test_sort (1764, 1026);
-  for (p = 2 ; p < 1764 ; p+=17)
-    for (n = 2 ; n < 1026 ; n+=42+p)
+  for (p = 2 ; p < 444 ; p += 17)
+    for (n = 2 ; n < 1026 ; n += 42 + p)
       test_sum (p, n);
 
   tests_end_mpfr ();
